﻿Imports Microsoft.Devices

Partial Public Class CamControl
    Inherits UserControl

#Region "Public"
    Public Event ActiveChanged(ByVal value As Boolean)
    Public Event RunChanged(ByVal value As Boolean)

    Private _Run As Boolean
    Public Property Run As Boolean
        Get
            Return _Run
        End Get
        Set(value As Boolean)
            If _Run <> value Then
                _Run = value
                Call StartCapture()
                RaiseEvent RunChanged(Me._Run)
            End If
        End Set
    End Property




#End Region

    ' Variables
    Private WithEvents Cam As Microsoft.Devices.PhotoCamera
    Private Shared PauseFramesEvent As New System.Threading.ManualResetEvent(True)
    Private thFrameWorker As System.Threading.Thread
    Private _FrameCheckActive As Boolean
    Private _bm As Imaging.WriteableBitmap
    Private NachScanner As AVB.NachtScanner

    Public Sub New()
        InitializeComponent()
        RaiseEvent ActiveChanged(False)
        NachScanner = New AVB.NachtScanner
    End Sub




    Private Sub CamControl_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded
        'Prüfen ob die Kamera verfügbar ist
        If (Microsoft.Devices.PhotoCamera.IsCameraTypeSupported(Microsoft.Devices.CameraType.Primary) = True Or
            Microsoft.Devices.PhotoCamera.IsCameraTypeSupported(Microsoft.Devices.CameraType.FrontFacing) = True) Then

            'Kamera initialisieren
            Me.Cam = New Microsoft.Devices.PhotoCamera()

            'Das Videobild an das Kontrol binden
            Me.viewfinderBrush.SetSource(Me.Cam)
            RaiseEvent ActiveChanged(True)
        Else
            'Die Kamera ist nicht unterstützt
            Me.Dispatcher.BeginInvoke(Sub()
                                          ' Write message.
                                          Me.txtDebug.Text = "An diesem Gerät ist keine Kamera verfügbar."
                                      End Sub)
            'Aktivierung unterdrücken
            RaiseEvent ActiveChanged(False)
        End If
    End Sub

    Private Sub Cam_CaptureImageAvailable(sender As Object, e As Microsoft.Devices.ContentReadyEventArgs) Handles Cam.CaptureImageAvailable

    End Sub

    Private Sub Cam_Initialized(sender As Object, e As Microsoft.Devices.CameraOperationCompletedEventArgs) Handles Cam.Initialized
        'Status der Kamera ausgeben
        If e.Succeeded = True Then
            Me.Dispatcher.BeginInvoke(Sub()
                                          If Me.Run Then Call StartCapture()
                                          Me.txtDebug.Text = "Kamera erfolgreich initialisiert"
                                      End Sub
                                      )
        Else
            Me.Dispatcher.BeginInvoke(Sub()
                                          Me.txtDebug.Text = "Fehler bei der Initialisierung der Kamera"
                                      End Sub)
        End If
    End Sub

    Private Sub StartCapture()
        ' Start ARGB to grayscale pump.
        'MainImage.Visibility = Visibility.Visible
        If _FrameCheckActive = True Then Exit Sub
        Me._FrameCheckActive = True


        ' Start pump.
        thFrameWorker = New System.Threading.Thread(AddressOf PumpARGBFrames)
        thFrameWorker.IsBackground = True
        thFrameWorker.Name = "thFrameWorker"
        thFrameWorker.Start()
        'Me.Dispatcher.BeginInvoke(Sub() txtDebug.Text = "ARGB to Grayscale")
    End Sub


    'ARGB frame pump
    Private Sub PumpARGBFrames()
        'Speicherbereich reservieren
        Dim ARGBPx(CInt(cam.PreviewResolution.Width) * CInt(cam.PreviewResolution.Height) - 1) As Integer

        Try
            Dim phCam As PhotoCamera = CType(cam, PhotoCamera)

            'Frames einlesen bis die Camera beendet wird
            Do While Me._FrameCheckActive

                'Warten bis die Daten bereit sind
                PauseFramesEvent.WaitOne()

                'Frame als Bytearray einlesen
                phCam.GetPreviewBufferArgb32(ARGBPx)

                'Bildauflösung auslesen
                Dim NewVideoSize As Size = phCam.PreviewResolution

                'Daten verarbeiten und Speicher klonen
                Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                              If Me._Run = True Then
                                                                  NachScanner.SetImageData(ARGBPx, NewVideoSize)
                                                              End If
                                                          End Sub)



                PauseFramesEvent.Reset()

                'Bildframe verändern und ausgeben
                Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                              If Me._bm Is Nothing Then
                                                                  Me._bm = New Imaging.WriteableBitmap(CInt(NewVideoSize.Width), CInt(NewVideoSize.Height))
                                                                  Me.MainImage.Source = Me._bm
                                                              End If

                                                              ARGBPx.CopyTo(_bm.Pixels, 0)
                                                              _bm.Invalidate()
                                                              PauseFramesEvent.Set()
                                                          End Sub)
            Loop

        Catch e As Exception

            ' Display error message.
            Me.Dispatcher.BeginInvoke(Sub()
                                          txtDebug.Text = e.Message
                                      End Sub)

        End Try

        Me._FrameCheckActive = False
    End Sub
End Class
